@@ -20,20 +20,19 @@
  * 
  *  This product is not supported/written/published by Apple!
  *
- *  $Id: db-artwork-debug.h 1299 2006-06-04 16:24:45Z jcsjcs $
+ *  $Id$
  */
 #ifndef DB_ARTWORK_DEBUG_H
 #define DB_ARTWORK_DEBUG_H
 
 #include "db-itunes-parser.h"
 
-/*#define DEBUG_ARTWORKDB*/
+/* #define DEBUG_ARTWORKDB */
 
 #ifdef DEBUG_ARTWORKDB
 extern G_GNUC_INTERNAL void dump_mhif (MhifHeader *mhif);
 extern G_GNUC_INTERNAL void dump_mhia (MhiaHeader *mhia);
-extern G_GNUC_INTERNAL void dump_mhod_type_1 (MhodHeaderArtworkType1 *mhod);
-extern G_GNUC_INTERNAL void dump_mhod_type_3 (ArtworkDB_MhodHeaderArtworkType3 *mhod);
+extern G_GNUC_INTERNAL void dump_mhod_string (ArtworkDB_MhodHeaderString *mhod);
 extern G_GNUC_INTERNAL void dump_mhni (MhniHeader *mhni);
 extern G_GNUC_INTERNAL void dump_mhod (ArtworkDB_MhodHeader *mhod);
 extern G_GNUC_INTERNAL void dump_mhii (MhiiHeader *mhii);
@@ -44,8 +43,7 @@ extern G_GNUC_INTERNAL void dump_mhba (MhbaHeader *mhba);
 #else 
 #define dump_mhif(x)
 #define dump_mhia(x)
-#define dump_mhod_type_1(x)
-#define dump_mhod_type_3(x)
+#define dump_mhod_string(x)
 #define dump_mhni(x)
 #define dump_mhod(x)
 #define dump_mhii(x)
