@@ -1,6 +1,5 @@
-/*  Time-stamp: <2006-11-11 20:38:32 jcs>
- *
- *  Copyright (C) 2005 Christophe Fergeau
+/*
+ *  Copyright (C) 2005-2007 Christophe Fergeau
  *
  * 
  *  The code contained in this file is free software; you can redistribute
@@ -21,7 +20,7 @@
  * 
  *  This product is not supported/written/published by Apple!
  *
- * $Id: db-itunes-parser.h,v 1.9 2006/11/11 13:44:29 jcsjcs Exp $
+ * $Id$
  */
 
 
@@ -29,9 +28,6 @@
 #define DB_PARSER_H
 
 #include <glib.h>
-/*#include "ipod-db-parser.h"*/
-
-#define ITUNESDB_MAX_SIZE 10 * 1024 * 1024
 
 struct _MHeader {
 	unsigned char header_id[4];
@@ -72,26 +68,44 @@ typedef struct _MhiaHeader MhiaHeader;
 
 typedef struct _MhitHeader471 MhitHeader471;
 /* MHOD typedef mess */
-typedef struct _ArtworkDB_MhodHeaderArtworkType3 ArtworkDB_MhodHeaderArtworkType3;
+typedef struct _ArtworkDB_MhodHeaderString ArtworkDB_MhodHeaderString;
 typedef struct _MhodHeaderString MhodHeaderString;
-typedef struct _MhodHeaderArtworkType1 MhodHeaderArtworkType1;
-/* typedef struct _MhodHeaderArtworkType3 MhodHeaderArtworkType3; */
 typedef struct _MhodHeaderSmartPlaylistData MhodHeaderSmartPlaylistData;
 typedef struct _MhodHeaderSmartPlaylistRuleString MhodHeaderSmartPlaylistRuleString; 
 typedef struct _MhodHeaderSmartPlaylistRuleNonString MhodHeaderSmartPlaylistRuleNonString;
 typedef struct _MhodHeaderSmartPlaylistRule MhodHeaderSmartPlaylistRule;
 
 struct _MhbdHeader {
-	unsigned char header_id[4];
-	gint32 header_len;
-	gint32 total_len;
-	gint32 unknown1;
-	gint32 version;
-	gint32 num_children;
-	gint64 db_id;
-	gint32 unknown2;
-	unsigned char padding[];
-};
+	gchar   header_id[4];
+	guint32 header_len;
+	guint32 total_len;
+	guint32 unknown1;
+	guint32 version;
+	guint32 num_children;
+	guint64 db_id;
+	guint16 platform;
+	guint16 unk_0x22;
+	guint64 id_0x24;
+	guint32 unk_0x2c;
+	guint16 hashing_scheme;
+	guchar  unk_0x32[20];
+	gchar   language_id[2];
+	guint64 db_persistent_id;
+	guint32 unk_0x50;
+	guint32 unk_0x54;
+	guchar  hash58[20];
+	gint32  timezone_offset;
+	guint16 unk_0x70;
+	guchar  hash72[46];
+	guint16 audio_language;
+	guint16 subtitle_language;
+	guint16 unk_0xa4;
+	guint16 unk_0xa6;
+	guint16 unk_0xa8;
+	guchar align_0xa9;
+	guchar hashAB[57];
+	guchar  padding[];
+} __attribute__((__packed__));
 
 enum MhsdIndexType {
 	MHSD_TRACK_LIST = 1,
@@ -159,7 +173,7 @@ struct _MhipHeader {
 	gint32 header_len;
 	gint32 total_len;
 	gint32 unknown1;
-	gint32 correlation_id;
+	gint32 format_id;
 	gint32 unknown2;
 	gint32 track_id;
 	gint32 timestamp;
@@ -363,54 +377,23 @@ enum MhodArtworkType {
 	MHOD_ARTWORK_TYPE_IMAGE      = 5  /* container: full resolution image (in the Photo Database) */
 };
 
-struct _MhodHeaderArtworkType1 {
+struct _ArtworkDB_MhodHeaderString {
 	unsigned char header_id[4];
 	gint32 header_len;
 	gint32 total_len;
-	gint16 type; 
-	gint8 unknown;
-	gint8 padding;
-	gint32 unknown1;
-	gint32 unknown2;
-	gint32 string_len;
-	gint32 unknown3; /* might be the string encoding */
-	gint32 unknown4; /* always zero? */
-	unsigned char string[];
-};
-
-struct _MhodHeaderArtworkType3 {
-	unsigned char header_id[4];
-	gint32 header_len;
-	gint32 total_len;
-	gint16 type; /* 3 */
-	gint16 padding; /* high byte is padding length (0-3) */
-	gint32 unknown1;
-	gint32 unknown2;
-	gint32 string_len;
-	gint32 mhod_version;
-	gint32 unknown4;
-	gunichar2 string[];
-};
-
-struct _ArtworkDB_MhodHeaderArtworkType3 {
-	unsigned char header_id[4];
-	gint32 header_len;
-	gint32 total_len;
-        gint16 type; /* 3 */
+        gint16 type;
         gint8  unknown13;
-	gint8  padding;
+	gint8  padding_len;
 	gint32 unknown1;
 	gint32 unknown2;
 	gint32 string_len;
-	gint8 mhod_version; /* 0,1: string is UTF8, 2: string is
-			       UTF16-LE */
+	gint8 encoding; /* 0,1: string is UTF8, 2: string is UTF16-LE */
         gint8 unknown5;
         gint16 unknown6;
 	gint32 unknown4;
         gchar  string[];
 };
 
-
 enum MhodLimitType {
 	MHOD_LIMIT_MINUTES = 1,
 	MHOD_LIMIT_MEGABYTES = 2,
@@ -543,7 +526,10 @@ struct _MhfdHeader {
 	gint32 header_len;
 	gint32 total_len;
 	gint32 unknown1;
-	gint32 unknown2;
+	gint32 unknown2; /* always seem to be 1 for older databases,
+			    in an ArtworkDB generated by iTunes 4.9 or
+			    above, it's 2. Caution: iTunes7 removes
+			    the whole database if this field is 1 */
 	gint32 num_children;
 	gint32 unknown3;
 	gint32 next_id;
@@ -593,7 +579,7 @@ struct _MhniHeader {
 	gint32 header_len;
 	gint32 total_len;
 	gint32 num_children;
-	gint32 correlation_id;
+	gint32 format_id;
 	gint32 ithmb_offset;
 	gint32 image_size;
 	gint16 vertical_padding;
@@ -626,13 +612,13 @@ struct _MhbaHeader {
 	guint8 repeat;     /* repeat the slideshow (from iPhoto setting) */
 	guint8 random;     /* show the slides in random order (from iPhoto
 			      setting) */
-        guint8 show_titles;/* show slide captions (from iPhoto setting) */
+	guint8 show_titles;/* show slide captions (from iPhoto setting) */
 	guint8 transition_direction; /* 0=none, 1=left-to-right,
 			      2=right-to-left, 3=top-to-bottom,
 			      4=bottom-to-top (from iPhoto setting) */
-        gint32 slide_duration; /* in seconds (from iPhoto setting) */
+	gint32 slide_duration; /* in seconds (from iPhoto setting) */
 	gint32 transition_duration; /* in milliseconds (from iPhoto setting) */
-        gint32 unk044;     /* unknown, seems to always be 0 */
+	gint32 unk044;     /* unknown, seems to always be 0 */
 	gint32 unk048;     /* unknown, seems to always be 0 */
 	gint64 song_id;    /* dbid2 of track in iTunesDB to play during
 			      slideshow (from iPhoto setting) */
@@ -652,7 +638,7 @@ struct _MhifHeader {
 	gint32 header_len;
 	gint32 total_len;
 	gint32 unknown1;
-	gint32 correlation_id;
+	gint32 format_id;
 	gint32 image_size;
 	unsigned char padding[];
 };
