@@ -1,7 +1,7 @@
 /*
 |  Copyright (C) 2002-2005 Jorg Schuler <jcsjcs at users sourceforge net>
 |  Part of the gtkpod project.
-| 
+|
 |  URL: http://www.gtkpod.org/
 |  URL: http://gtkpod.sourceforge.net/
 |
@@ -23,13 +23,14 @@
 |
 |  This product is not supported/written/published by Apple!
 |
-|  $Id: itdb_artwork.c 1753 2007-11-04 00:48:12Z jcsjcs $
+|  $Id$
 */
 
 #include <config.h>
 
 #include "itdb_device.h"
 #include "itdb_private.h"
+#include "itdb_thumb.h"
 #include "db-image-parser.h"
 #include "itdb_endianness.h"
 #include <errno.h>
@@ -44,12 +45,14 @@
 
 /**
  * itdb_artwork_new:
- * 
- * Creates a new #Itdb_Artwork 
  *
- * Return value: a new #Itdb_Artwork to be freed with itdb_artwork_free() when
+ * Creates a new #Itdb_Artwork
+ *
+ * Returns: a new #Itdb_Artwork to be freed with itdb_artwork_free() when
  * no longer needed
- **/
+ *
+ * Since: 0.3.0
+ */
 Itdb_Artwork *itdb_artwork_new (void)
 {
     Itdb_Artwork *artwork = g_new0 (Itdb_Artwork, 1);
@@ -61,7 +64,9 @@ Itdb_Artwork *itdb_artwork_new (void)
  * @artwork: an #Itdb_Artwork
  *
  * Frees memory used by @artwork
- **/
+ *
+ * Since: 0.3.0
+ */
 void itdb_artwork_free (Itdb_Artwork *artwork)
 {
     g_return_if_fail (artwork);
@@ -71,37 +76,16 @@ void itdb_artwork_free (Itdb_Artwork *artwork)
     g_free (artwork);
 }
 
-
-static GList *dup_thumbnails (GList *thumbnails)
-{
-    GList *it;
-    GList *result;
-    
-    result = NULL;
-    for (it = thumbnails; it != NULL; it = it->next)
-    {
-	Itdb_Thumb *new_thumb;
-	Itdb_Thumb *thumb;
-
-	thumb = (Itdb_Thumb *)it->data;
-	g_return_val_if_fail (thumb, NULL);
-
-	new_thumb = itdb_thumb_duplicate (thumb);
-	
-	result = g_list_prepend (result, new_thumb);
-    }
-
-    return g_list_reverse (result);
-}
-
 /**
  * itdb_artwork_duplicate:
  * @artwork: an #Itdb_Artwork
  *
  * Duplicates @artwork
  *
- * Return value: a new copy of @artwork
- **/
+ * Returns: a new copy of @artwork
+ *
+ * Since: 0.3.0
+ */
 Itdb_Artwork *itdb_artwork_duplicate (Itdb_Artwork *artwork)
 {
     Itdb_Artwork *dup;
@@ -111,77 +95,57 @@ Itdb_Artwork *itdb_artwork_duplicate (Itdb_Artwork *artwork)
 
     memcpy (dup, artwork, sizeof (Itdb_Artwork));
 
-    dup->thumbnails = dup_thumbnails (artwork->thumbnails);
+    if (artwork->thumbnail != NULL) {
+        dup->thumbnail = itdb_thumb_duplicate (artwork->thumbnail);
+    }
 
     return dup;
 }
 
-
-/**
- * itdb_artwork_remove_thumbnail:
- * @artwork: an #Itdb_Artwork
- * @thumb: an #Itdb_Thumb
- *
- * Removes @thumb from @artwork. The memory used by @thumb is freed.
- **/
-void
-itdb_artwork_remove_thumbnail (Itdb_Artwork *artwork, Itdb_Thumb *thumb)
-{
-    g_return_if_fail (artwork);
-    g_return_if_fail (thumb);
-
-    artwork->thumbnails = g_list_remove (artwork->thumbnails, thumb);
-    itdb_thumb_free (thumb);
-}
-
-
 /**
  * itdb_artwork_remove_thumbnails:
  * @artwork: an #Itdb_Artwork
  *
- * Removes all thumbnails from @artwork 
- **/
+ * Removes all thumbnails from @artwork
+ *
+ * Since: 0.3.0
+ */
 void
 itdb_artwork_remove_thumbnails (Itdb_Artwork *artwork)
 {
     g_return_if_fail (artwork);
 
-    while (artwork->thumbnails)
-    {
-	Itdb_Thumb *thumb = artwork->thumbnails->data;
-	g_return_if_fail (thumb);
-	itdb_artwork_remove_thumbnail (artwork, thumb);
+    if (artwork->thumbnail != NULL) {
+        itdb_thumb_free (artwork->thumbnail);
     }
+    artwork->thumbnail = NULL;
     artwork->artwork_size = 0;
     artwork->id = 0;
 }
 
-
-
-
 /**
- * itdb_artwork_add_thumbnail
- * @artwork: an #Itdb_Thumbnail
- * @type: thumbnail size
- * @filename: image file to use to create the thumbnail
- * @rotation: angle by which the image should be rotated
- * counterclockwise. Valid values are 0, 90, 180 and 270.
- * @error: return location for a #GError or NULL
+ * itdb_artwork_set_thumbnail
+ * @artwork:    an #Itdb_Artwork
+ * @filename:   image file to use to create the thumbnail
+ * @rotation:   angle by which the image should be rotated
+ *              counterclockwise. Valid values are 0, 90, 180 and 270.
+ * @error:      return location for a #GError or NULL
  *
- * Appends a thumbnail of type @type to existing thumbnails in @artwork. No 
+ * Appends a thumbnail of type @type to existing thumbnails in @artwork. No
  * data is read from @filename yet, the file will be when @artwork is saved to
  * disk. @filename must still exist when that happens.
  *
  * For the rotation angle you can also use the gdk constants
- * GDK_PIXBUF_ROTATE_NONE, ..._COUNTERCLOCKWISE, ..._UPSIDEDOWN AND
- * ..._CLOCKWISE.
+ * %GDK_PIXBUF_ROTATE_NONE, %GDK_PIXBUF_ROTATE_COUNTERCLOCKWISE,
+ * %GDK_PIXBUF_ROTATE_UPSIDEDOWN, AND %GDK_PIXBUF_ROTATE_CLOCKWISE.
  *
- * Return value: TRUE if the thumbnail could be successfully added, FALSE
+ * Returns: TRUE if the thumbnail could be successfully added, FALSE
  * otherwise. @error is set appropriately.
- **/
+ *
+ * Since: 0.7.0
+ */
 gboolean
-itdb_artwork_add_thumbnail (Itdb_Artwork *artwork,
-			    ItdbThumbType type,
+itdb_artwork_set_thumbnail (Itdb_Artwork *artwork,
 			    const gchar *filename,
 			    gint rotation,
 			    GError **error)
@@ -204,11 +168,12 @@ itdb_artwork_add_thumbnail (Itdb_Artwork *artwork,
     artwork->artwork_size  = statbuf.st_size;
     artwork->creation_date = statbuf.st_mtime;
 
-    thumb = itdb_thumb_new ();
-    thumb->filename = g_strdup (filename);
-    thumb->type = type;
-    thumb->rotation = rotation;
-    artwork->thumbnails = g_list_append (artwork->thumbnails, thumb);
+    thumb = itdb_thumb_new_from_file (filename);
+    itdb_thumb_set_rotation (thumb, rotation);
+    if (artwork->thumbnail != NULL) {
+        itdb_thumb_free (artwork->thumbnail);
+    }
+    artwork->thumbnail = thumb;
 
     return TRUE;
 #else
@@ -219,29 +184,28 @@ itdb_artwork_add_thumbnail (Itdb_Artwork *artwork,
 }
 
 /**
- * itdb_artwork_add_thumbnail_from_pixbuf
- * @artwork: an #Itdb_Thumbnail
- * @type: thumbnail size
- * @pixbuf: #GdkPixbuf to use to create the thumbnail
- * @rotation: angle by which the image should be rotated
- * counterclockwise. Valid values are 0, 90, 180 and 270.
- * @error: return location for a #GError or NULL
+ * itdb_artwork_set_thumbnail_from_pixbuf
+ * @artwork:    an #Itdb_Artwork
+ * @pixbuf:     #GdkPixbuf to use to create the thumbnail
+ * @rotation:   angle by which the image should be rotated
+ *              counterclockwise. Valid values are 0, 90, 180 and 270.
+ * @error:      return location for a #GError or NULL
  *
- * Appends a thumbnail of type @type to existing thumbnails in @artwork. No 
- * data is generated from @pixbuf yet, it will be done when @artwork is saved 
- * to disk. @pixbuf is ref'ed by this function, but is not copied, so it should
- * not be modified before the database is saved.
+ * Set a thumbnail in @artwork. No data is generated from @pixbuf yet, it will
+ * be done when @artwork is saved to disk. @pixbuf is ref'ed by this function,
+ * but is not copied, so it should not be modified before the database is saved.
  *
  * For the rotation angle you can also use the gdk constants
- * GDK_PIXBUF_ROTATE_NONE, ..._COUNTERCLOCKWISE, ..._UPSIDEDOWN AND
- * ..._CLOCKWISE.
+ * %GDK_PIXBUF_ROTATE_NONE, %GDK_PIXBUF_ROTATE_COUNTERCLOCKWISE,
+ * %GDK_PIXBUF_ROTATE_UPSIDEDOWN, AND %GDK_PIXBUF_ROTATE_CLOCKWISE.
  *
- * Return value: TRUE if the thumbnail could be successfully added, FALSE
+ * Returns: TRUE if the thumbnail could be successfully added, FALSE
  * otherwise. @error is set appropriately.
- **/
+ *
+ * Since: 0.7.0
+ */
 gboolean
-itdb_artwork_add_thumbnail_from_pixbuf (Itdb_Artwork *artwork,
-                                        ItdbThumbType type,
+itdb_artwork_set_thumbnail_from_pixbuf (Itdb_Artwork *artwork,
                                         gpointer pixbuf,
                                         gint rotation,
                                         GError **error)
@@ -264,12 +228,12 @@ itdb_artwork_add_thumbnail_from_pixbuf (Itdb_Artwork *artwork,
     artwork->artwork_size  = rowstride * height;
     artwork->creation_date = time.tv_sec;
 
-    thumb = itdb_thumb_new ();
-    g_object_ref (G_OBJECT (pixbuf));
-    thumb->pixbuf = pixbuf;
-    thumb->type = type;
-    thumb->rotation = rotation;
-    artwork->thumbnails = g_list_append (artwork->thumbnails, thumb);
+    thumb = itdb_thumb_new_from_pixbuf (pixbuf);
+    itdb_thumb_set_rotation (thumb, rotation);
+    if (artwork->thumbnail != NULL) {
+        itdb_thumb_free (artwork->thumbnail);
+    }
+    artwork->thumbnail = thumb;
 
     return TRUE;
 #else
@@ -280,30 +244,29 @@ itdb_artwork_add_thumbnail_from_pixbuf (Itdb_Artwork *artwork,
 }
 
 /**
- * itdb_artwork_add_thumbnail_from_data
- * @artwork: an #Itdb_Thumbnail
- * @type: thumbnail size
- * @image_data: data used to create the thumbnail (the raw contents of
- *              an image file)
+ * itdb_artwork_set_thumbnail_from_data
+ * @artwork:        an #Itdb_Artwork
+ * @image_data:     data used to create the thumbnail (the raw contents of
+ *                  an image file)
  * @image_data_len: length of above data block
- * @rotation: angle by which the image should be rotated
- * counterclockwise. Valid values are 0, 90, 180 and 270.
- * @error: return location for a #GError or NULL
+ * @rotation:       angle by which the image should be rotated
+ *                  counterclockwise. Valid values are 0, 90, 180 and 270.
+ * @error:          return location for a #GError or NULL
  *
- * Appends a thumbnail of type @type to existing thumbnails in
- * @artwork. No data is processed yet. This will be done when @artwork
- * is saved to disk.
+ * Set a thumbnail in @artwork. No data is processed yet. This will be done when
+ * @artwork is saved to disk.
  *
  * For the rotation angle you can also use the gdk constants
- * GDK_PIXBUF_ROTATE_NONE, ..._COUNTERCLOCKWISE, ..._UPSIDEDOWN AND
- * ..._CLOCKWISE.
+ * %GDK_PIXBUF_ROTATE_NONE, %GDK_PIXBUF_ROTATE_COUNTERCLOCKWISE,
+ * %GDK_PIXBUF_ROTATE_UPSIDEDOWN, AND %GDK_PIXBUF_ROTATE_CLOCKWISE.
  *
- * Return value: TRUE if the thumbnail could be successfully added, FALSE
+ * Returns: TRUE if the thumbnail could be successfully added, FALSE
  * otherwise. @error is set appropriately.
- **/
+ *
+ * Since: 0.7.0
+ */
 gboolean
-itdb_artwork_add_thumbnail_from_data (Itdb_Artwork *artwork,
-				      ItdbThumbType type,
+itdb_artwork_set_thumbnail_from_data (Itdb_Artwork *artwork,
 				      const guchar *image_data,
 				      gsize image_data_len,
 				      gint rotation,
@@ -322,14 +285,12 @@ itdb_artwork_add_thumbnail_from_data (Itdb_Artwork *artwork,
     artwork->artwork_size  = image_data_len;
     artwork->creation_date = time.tv_sec;
 
-    thumb = itdb_thumb_new ();
-    thumb->image_data = g_malloc (image_data_len);
-    thumb->image_data_len = image_data_len;
-    memcpy (thumb->image_data,  image_data, image_data_len);
-
-    thumb->type = type;
-    thumb->rotation = rotation;
-    artwork->thumbnails = g_list_append (artwork->thumbnails, thumb);
+    thumb = itdb_thumb_new_from_data (image_data, image_data_len);
+    itdb_thumb_set_rotation (thumb, rotation);
+    if (artwork->thumbnail != NULL) {
+        itdb_thumb_free (artwork->thumbnail);
+    }
+    artwork->thumbnail = thumb;
 
     return TRUE;
 #else
@@ -339,93 +300,6 @@ itdb_artwork_add_thumbnail_from_data (Itdb_Artwork *artwork,
 #endif
 }
 
-
-/**
- * itdb_artwork_get_thumb_by_type:
- * @artwork: an #Itdb_Artwork
- * @type: type of the #Itdb_Thumb to retrieve
- *
- * Searches @artwork for an #Itdb_Thumb of type @type.
- * 
- * Returns: an #Itdb_Thumb of type @type, or NULL if such a thumbnail couldn't
- * be found
- **/
-Itdb_Thumb *itdb_artwork_get_thumb_by_type (Itdb_Artwork *artwork,
-					    ItdbThumbType type)
-{
-    GList *gl;
-
-    g_return_val_if_fail (artwork, NULL);
-
-    for (gl=artwork->thumbnails; gl; gl=gl->next)
-    {
-	Itdb_Thumb *thumb = gl->data;
-	g_return_val_if_fail (thumb, NULL);
-	if (thumb->type == type)  return thumb;
-    }
-    return NULL;
-}
-
-
-/**
- * itdb_thumb_get_filename:
- * @device: an #Itdb_Device
- * @thumb: an #Itdb_Thumb
- *
- * Get filename of thumbnail. If it's a thumbnail on the iPod, return
- * the full path to the ithmb file. Otherwise return the full path to
- * the original file.
- *
- * Return value: newly allocated string containing the absolute path to the 
- * thumbnail file. 
- **/
-gchar *itdb_thumb_get_filename (Itdb_Device *device, Itdb_Thumb *thumb)
-{
-    gchar *artwork_dir, *filename=NULL;
-
-    g_return_val_if_fail (device, NULL);
-    g_return_val_if_fail (thumb, NULL);
-
-    /* thumbnail not transferred to the iPod */
-    if (thumb->size == 0)
-	return g_strdup (thumb->filename);
-
-    if (strlen (thumb->filename) < 2)
-    {
-	g_print (_("Illegal filename: '%s'.\n"), thumb->filename);
-	return NULL;
-    }
-
-    if (!device->mountpoint)
-    {
-	g_print (_("Mountpoint not set.\n"));
-	return NULL;
-    }
-    artwork_dir = itdb_get_artwork_dir (device->mountpoint);
-    if (artwork_dir)
-    {
-	filename = itdb_get_path (artwork_dir, thumb->filename+1);
-	g_free (artwork_dir);
-    }
-    /* FIXME: Hack */
-    if( !filename ) {
-	artwork_dir = itdb_get_photos_thumb_dir (device->mountpoint);
-
-	if (artwork_dir)
-	{
-	    const gchar *name_on_disk = strchr( thumb->filename+1, ':');
-	    if (name_on_disk)
-	    {
-		filename = itdb_get_path (artwork_dir, name_on_disk + 1);
-	    }
-	    g_free (artwork_dir);
-	}
-
-    }
-    return filename;
-}
-
-
 #if HAVE_GDKPIXBUF
 static guchar *
 unpack_RGB_565 (guint16 *pixels, guint bytes_len, guint byte_order)
@@ -445,7 +319,7 @@ unpack_RGB_565 (guint16 *pixels, guint bytes_len, guint byte_order)
 		result[3*i] = (cur_pixel & RED_MASK_565) >> RED_SHIFT_565;
 		result[3*i+1] = (cur_pixel & GREEN_MASK_565) >> GREEN_SHIFT_565;
 		result[3*i+2] = (cur_pixel & BLUE_MASK_565) >> BLUE_SHIFT_565;
-		
+
 		/* Normalize color values so that they use a [0..255] range */
 		result[3*i] <<= (8 - RED_BITS_565);
 		result[3*i+1] <<= (8 - GREEN_BITS_565);
@@ -473,7 +347,7 @@ unpack_RGB_555 (guint16 *pixels, guint bytes_len, guint byte_order)
 		result[3*i] = (cur_pixel & RED_MASK_555) >> RED_SHIFT_555;
 		result[3*i+1] = (cur_pixel & GREEN_MASK_555) >> GREEN_SHIFT_555;
 		result[3*i+2] = (cur_pixel & BLUE_MASK_555) >> BLUE_SHIFT_555;
-		
+
 		/* Normalize color values so that they use a [0..255] range */
 		result[3*i] <<= (8 - RED_BITS_555);
 		result[3*i+1] <<= (8 - GREEN_BITS_555);
@@ -502,7 +376,7 @@ unpack_RGB_888 (guint16 *pixels, guint bytes_len, guint byte_order)
 		result[3*i] = (cur_pixel & RED_MASK_888) >> RED_SHIFT_888;
 		result[3*i+1] = (cur_pixel & GREEN_MASK_888) >> GREEN_SHIFT_888;
 		result[3*i+2] = (cur_pixel & BLUE_MASK_888) >> BLUE_SHIFT_888;
-		
+
 		/* Normalize color values so that they use a [0..255] range */
 		/* (not necessary for 888 encoding) */
 /* 		result[3*i] <<= (8 - RED_BITS_888); */
@@ -547,7 +421,7 @@ static guint16 *rearrange_pixels (guint16 *pixels_s, guint16 *pixels_d,
 			  width/2, height/2,
 			  row_stride);
     }
-    
+
     return pixels_d;
 }
 
@@ -561,8 +435,6 @@ unpack_rec_RGB_555 (guint16 *pixels, guint bytes_len, guint byte_order,
 	gboolean free_use_pixels = FALSE;
 	guint16 *pixels_arranged;
 
-	guint i;
-
 	g_return_val_if_fail (bytes_len < 2*(G_MAXUINT/3), NULL);
 	g_return_val_if_fail (2*width*height < G_MAXUINT, NULL);
 	g_return_val_if_fail (width==height, NULL);
@@ -586,22 +458,7 @@ unpack_rec_RGB_555 (guint16 *pixels, guint bytes_len, guint byte_order,
 	    return NULL;
 	}
 
-	result = g_malloc ((bytes_len/2) * 3);
-
-	for (i = 0; i < bytes_len/2; i++) {
-		guint16 cur_pixel;
-		/* FIXME: endianness */
-		cur_pixel = get_gint16 (pixels_arranged[i], byte_order);
-		/* Unpack pixels */
-		result[3*i] = (cur_pixel & RED_MASK_555) >> RED_SHIFT_555;
-		result[3*i+1] = (cur_pixel & GREEN_MASK_555) >> GREEN_SHIFT_555;
-		result[3*i+2] = (cur_pixel & BLUE_MASK_555) >> BLUE_SHIFT_555;
-		
-		/* Normalize color values so that they use a [0..255] range */
-		result[3*i] <<= (8 - RED_BITS_555);
-		result[3*i+1] <<= (8 - GREEN_BITS_555);
-		result[3*i+2] <<= (8 - BLUE_BITS_555);
-	}
+	result = unpack_RGB_555 (pixels_arranged, bytes_len, byte_order);
 
 	g_free (pixels_arranged);
 	if (free_use_pixels)
@@ -637,7 +494,7 @@ printf ("%8x\n", cur_pixel);
 		result[3*i] = (cur_pixel & RED_MASK_888) >> RED_SHIFT_888;
 		result[3*i+1] = (cur_pixel & GREEN_MASK_888) >> GREEN_SHIFT_888;
 		result[3*i+2] = (cur_pixel & BLUE_MASK_888) >> BLUE_SHIFT_888;
-		
+
 		/* Normalize color values so that they use a [0..255] range */
 		/* (not really necessary for 888 encoding) */
 /* 		result[3*i] <<= (8 - RED_BITS_888); */
@@ -695,9 +552,9 @@ unpack_I420 (guchar *yuvdata, gint bytes_len, guint byte_order,
 		u = yuvdata[ustart + (row/2)*(width/2) + col/2];
 		v = yuvdata[vstart + (row/2)*(width/2) + col/2];
 
-		rgbdata[z] = limit8bit((y-16)*1.164 + (v-128)*1.596); 
-		rgbdata[z+1] = limit8bit((y-16)*1.164 - (v-128)*0.813 - (u-128)*0.391); 
-		rgbdata[z+2] = limit8bit((y-16)*1.164 + (u-128)*2.018); 
+		rgbdata[z] = limit8bit((y-16)*1.164 + (v-128)*1.596);
+		rgbdata[z+1] = limit8bit((y-16)*1.164 - (v-128)*0.813 - (u-128)*0.391);
+		rgbdata[z+2] = limit8bit((y-16)*1.164 + (u-128)*2.018);
 
 		z+=3;
 		h++;
@@ -772,7 +629,7 @@ unpack_UYVY (guchar *yuvdata, gint bytes_len, guint byte_order,
 }
 
 static guchar *
-get_pixel_data (Itdb_Device *device, Itdb_Thumb *thumb)
+get_pixel_data (Itdb_Device *device, Itdb_Thumb_Ipod_Item *thumb)
 {
 	gchar *filename = NULL;
 	guchar *result = NULL;
@@ -787,7 +644,7 @@ get_pixel_data (Itdb_Device *device, Itdb_Thumb *thumb)
 	 */
 	result = g_malloc (thumb->size);
 
-	filename = itdb_thumb_get_filename (device, thumb);
+	filename = itdb_thumb_ipod_get_filename (device, thumb);
 
 	if (!filename)
 	{
@@ -798,7 +655,7 @@ get_pixel_data (Itdb_Device *device, Itdb_Thumb *thumb)
 
 	f = fopen (filename, "r");
 	if (f == NULL) {
-		g_print ("Failed to open %s: %s\n", 
+		g_print ("Failed to open %s: %s\n",
 			 filename, strerror (errno));
 		goto error;
 	}
@@ -812,7 +669,7 @@ get_pixel_data (Itdb_Device *device, Itdb_Thumb *thumb)
 
 	res = fread (result, thumb->size, 1, f);
 	if (res != 1) {
-		g_print ("Failed to read %u bytes from %s: %s\n", 
+		g_print ("Failed to read %u bytes from %s: %s\n",
 			 thumb->size, thumb->filename, strerror (errno));
 		goto error;
 	}
@@ -832,7 +689,7 @@ get_pixel_data (Itdb_Device *device, Itdb_Thumb *thumb)
 }
 
 static guchar *
-itdb_thumb_get_rgb_data (Itdb_Device *device, Itdb_Thumb *thumb)
+itdb_thumb_get_rgb_data (Itdb_Device *device, Itdb_Thumb_Ipod_Item *item)
 {
 #if 0
     #include <unistd.h>
@@ -843,15 +700,16 @@ itdb_thumb_get_rgb_data (Itdb_Device *device, Itdb_Thumb *thumb)
 #endif
 	void *pixels_raw;
 	guchar *pixels=NULL;
-	const Itdb_ArtworkFormat *img_info;
 
 	g_return_val_if_fail (device, NULL);
-	g_return_val_if_fail (thumb, NULL);
-	g_return_val_if_fail (thumb->size != 0, NULL);
-	img_info = itdb_get_artwork_info_from_type (device, thumb->type);
-	g_return_val_if_fail (img_info, NULL);
-	
-	pixels_raw = get_pixel_data (device, thumb);
+	g_return_val_if_fail (item, NULL);
+	g_return_val_if_fail (item->size != 0, NULL);
+
+        if (item->format == NULL) {
+            return NULL;
+        }
+
+	pixels_raw = get_pixel_data (device, item);
 
 #if 0
     name = g_strdup_printf ("thumb_%03d.raw", i++);
@@ -864,7 +722,7 @@ itdb_thumb_get_rgb_data (Itdb_Device *device, Itdb_Thumb *thumb)
 		return NULL;
 	}
 
-	switch (img_info->format)
+	switch (item->format->format)
 	{
 	case THUMB_FORMAT_RGB565_LE_90:
 	case THUMB_FORMAT_RGB565_BE_90:
@@ -873,8 +731,8 @@ itdb_thumb_get_rgb_data (Itdb_Device *device, Itdb_Thumb *thumb)
 	       screen photo thumbnail) */
 	case THUMB_FORMAT_RGB565_LE:
 	case THUMB_FORMAT_RGB565_BE:
-	    pixels = unpack_RGB_565 (pixels_raw, thumb->size,
-				     itdb_thumb_get_byteorder (img_info->format));
+	    pixels = unpack_RGB_565 (pixels_raw, item->size,
+				     itdb_thumb_get_byteorder (item->format->format));
 	    break;
 	case THUMB_FORMAT_RGB555_LE_90:
 	case THUMB_FORMAT_RGB555_BE_90:
@@ -883,8 +741,8 @@ itdb_thumb_get_rgb_data (Itdb_Device *device, Itdb_Thumb *thumb)
 	       screen photo thumbnail) */
 	case THUMB_FORMAT_RGB555_LE:
 	case THUMB_FORMAT_RGB555_BE:
-	    pixels = unpack_RGB_555 (pixels_raw, thumb->size,
-				     itdb_thumb_get_byteorder (img_info->format));
+	    pixels = unpack_RGB_555 (pixels_raw, item->size,
+				     itdb_thumb_get_byteorder (item->format->format));
 	    break;
 	case THUMB_FORMAT_RGB888_LE_90:
 	case THUMB_FORMAT_RGB888_BE_90:
@@ -892,8 +750,8 @@ itdb_thumb_get_rgb_data (Itdb_Device *device, Itdb_Thumb *thumb)
 	       different treatment */
 	case THUMB_FORMAT_RGB888_LE:
 	case THUMB_FORMAT_RGB888_BE:
-	    pixels = unpack_RGB_888 (pixels_raw, thumb->size,
-				     itdb_thumb_get_byteorder (img_info->format));
+	    pixels = unpack_RGB_888 (pixels_raw, item->size,
+				     itdb_thumb_get_byteorder (item->format->format));
 	    break;
 	case THUMB_FORMAT_REC_RGB555_LE_90:
 	case THUMB_FORMAT_REC_RGB555_BE_90:
@@ -902,29 +760,29 @@ itdb_thumb_get_rgb_data (Itdb_Device *device, Itdb_Thumb *thumb)
 	       screen photo thumbnail) */
 	case THUMB_FORMAT_REC_RGB555_LE:
 	case THUMB_FORMAT_REC_RGB555_BE:
-	    pixels = unpack_rec_RGB_555 (pixels_raw, thumb->size,
-					 itdb_thumb_get_byteorder (img_info->format),
-					 img_info->width, img_info->height);
+	    pixels = unpack_rec_RGB_555 (pixels_raw, item->size,
+					 itdb_thumb_get_byteorder (item->format->format),
+					 item->format->width, item->format->height);
 	    break;
 	case THUMB_FORMAT_EXPERIMENTAL_LE:
 	case THUMB_FORMAT_EXPERIMENTAL_BE:
 #if DEBUG_ARTWORK
-	    pixels = unpack_experimental (pixels_raw, thumb->size,
-					  itdb_thumb_get_byteorder (img_info->format),
-					  img_info->width, img_info->height);
+	    pixels = unpack_experimental (pixels_raw, item->size,
+					  itdb_thumb_get_byteorder (item->format->format),
+					  item->format->width, item->format->height);
 	    break;
 #endif
 	case THUMB_FORMAT_UYVY_LE:
 	case THUMB_FORMAT_UYVY_BE:
-	    pixels = unpack_UYVY (pixels_raw, thumb->size,
-				  itdb_thumb_get_byteorder (img_info->format),
-				  img_info->width, img_info->height);
+	    pixels = unpack_UYVY (pixels_raw, item->size,
+				  itdb_thumb_get_byteorder (item->format->format),
+				  item->format->width, item->format->height);
 	    break;
 	case THUMB_FORMAT_I420_LE:
 	case THUMB_FORMAT_I420_BE:
-	    pixels = unpack_I420 (pixels_raw, thumb->size,
-				  itdb_thumb_get_byteorder (img_info->format),
-				  img_info->width, img_info->height);
+	    pixels = unpack_I420 (pixels_raw, item->size,
+				  itdb_thumb_get_byteorder (item->format->format),
+				  item->format->width, item->format->height);
 	    break;
 	}
 	g_free (pixels_raw);
@@ -932,169 +790,66 @@ itdb_thumb_get_rgb_data (Itdb_Device *device, Itdb_Thumb *thumb)
 	return pixels;
 
 }
-#endif
 
-
-
-/**
- * itdb_thumb_get_gdk_pixbuf:
- * @device: an #Itdb_Device
- * @thumb: an #Itdb_Thumb
- * 
- * Converts @thumb to a #GdkPixbuf.
- * Since we want to have gdk-pixbuf dependency optional, a generic
- * gpointer is returned which you have to cast to a #GdkPixbuf using 
- * GDK_PIXBUF() yourself. 
- *
- * Return value: a #GdkPixbuf that must be unreffed with gdk_pixbuf_unref()
- * after use, or NULL if the creation of the gdk-pixbuf failed or if 
- * libgpod was compiled without gdk-pixbuf support.
- **/
-gpointer
-itdb_thumb_get_gdk_pixbuf (Itdb_Device *device, Itdb_Thumb *thumb)
+static guint get_aligned_width (const Itdb_ArtworkFormat *img_info,
+                                gsize pixel_size)
 {
-#if HAVE_GDKPIXBUF
-    GdkPixbuf *pixbuf=NULL;
-    guchar *pixels;
-    const Itdb_ArtworkFormat *img_info=NULL;
+    guint width;
+    guint alignment = img_info->row_bytes_alignment/pixel_size;
 
-    g_return_val_if_fail (thumb, NULL);
-
-    /* If we are dealing with an iPod (device != NULL), use default
-       image dimensions as used on the iPod in question.
-
-       If we are not dealing with an iPod, we can only return a pixmap
-       for thumbnails "not transferred to the iPod" (thumb->size ==
-       0).
-    */
-    if (device != NULL)
-    {
-	img_info = itdb_get_artwork_info_from_type (device, thumb->type);
+    if (alignment * pixel_size != img_info->row_bytes_alignment) {
+        g_warning ("RowBytesAlignment (%d) not a multiple of pixel size (%"G_GSIZE_FORMAT")",
+                   img_info->row_bytes_alignment, pixel_size);
     }
 
-    if (thumb->size == 0)
-    {   /* thumbnail has not yet been transferred to the iPod */
-	gint width=0, height=0;
-
-	if (img_info != NULL)
-	{   /* use image dimensions from iPod */
-	    width = img_info->width;
-	    height = img_info->height;
-	}
-	else
-	{   /* use default dimensions */
-	    /* FIXME: better way to use the ipod_color dimensions? */
-	    switch (thumb->type)
-	    {
-	    case ITDB_THUMB_COVER_SMALL:
-		width =  56;  height =  56;  break;
-	    case ITDB_THUMB_COVER_LARGE:
-		width = 140;  height = 140;  break;
-	    case ITDB_THUMB_PHOTO_SMALL:
-		width =  42;  height =  30;  break;
-	    case ITDB_THUMB_PHOTO_LARGE:
-		width = 130;  height =  88;  break;
-	    case ITDB_THUMB_PHOTO_FULL_SCREEN:
-		width = 220;  height = 176;  break;
-	    case ITDB_THUMB_PHOTO_TV_SCREEN:
-		width = 720;  height = 480;  break;
-	    case ITDB_THUMB_COVER_XLARGE:
-		width = 320;  height = 320;  break;
-	    case ITDB_THUMB_COVER_MEDIUM:
-		width = 128;  height = 128;  break;
-	    case ITDB_THUMB_COVER_SMEDIUM:
-		width = 88;  height = 88;  break;
-	    case ITDB_THUMB_COVER_XSMALL:
-		width = 56;  height = 56;  break;
-	    }
-	    if (width == 0)
-	    {
-		width = 140;
-		height = 140;
-	    }
-	}
-
-	if (thumb->filename)
-	{   /* read data from filename */
-	    pixbuf = gdk_pixbuf_new_from_file_at_size (thumb->filename, 
-						       width, height,
-						       NULL);
-	}
-	else if (thumb->image_data)
-	{   /* use data stored in image_data */
-	    	GdkPixbufLoader *loader = gdk_pixbuf_loader_new ();
-		g_return_val_if_fail (loader, FALSE);
-		gdk_pixbuf_loader_set_size (loader,
-					    width, height);
-		gdk_pixbuf_loader_write (loader,
-					 thumb->image_data,
-					 thumb->image_data_len,
-					 NULL);
-		gdk_pixbuf_loader_close (loader, NULL);
-		pixbuf = gdk_pixbuf_loader_get_pixbuf (loader);
-		if (pixbuf)
-		    g_object_ref (pixbuf);
-		g_object_unref (loader);
-	}
-	else if (thumb->pixbuf)
-	{   /* use pixbuf data */
-	    pixbuf = gdk_pixbuf_scale_simple (thumb->pixbuf,
-					      width, height,
-					      GDK_INTERP_BILINEAR);
-	}
-
-	if (!pixbuf)
-	{
-	    return NULL;
-	}
-
-	/* !! cannot write directly to &thumb->width/height because
-	   g_object_get() returns a gint, but thumb->width/height are
-	   gint16 !! */
-	g_object_get (G_OBJECT (pixbuf), 
-		      "width", &width,
-		      "height", &height,
-		      NULL);
-
-	thumb->width = width;
-	thumb->height = height;
+    width = img_info->width;
+    if ((alignment != 0) && ((img_info->width % alignment) != 0)) {
+        width += alignment - (img_info->width % alignment);
     }
-    else
-    {
+    return width;
+}
+
+gpointer itdb_thumb_ipod_item_to_pixbuf (Itdb_Device *device,
+                                         Itdb_Thumb_Ipod_Item *item)
+{
 	/* pixbuf is already on the iPod -> read from there */
 	GdkPixbuf *pixbuf_full;
 	GdkPixbuf *pixbuf_sub;
-	gint pad_x = thumb->horizontal_padding;
-	gint pad_y = thumb->vertical_padding;
-	gint width = thumb->width;
-	gint height = thumb->height;
+        GdkPixbuf *pixbuf;
+	gint pad_x = item->horizontal_padding;
+	gint pad_y = item->vertical_padding;
+	gint width = item->width;
+	gint height = item->height;
+	guint rowstride;
+        const Itdb_ArtworkFormat *img_info = item->format;
+        guchar *pixels;
 
 /*	printf ("hp%d vp%d w%d h%d\n",
 	       pad_x, pad_y, width, height);*/
-
-	if (img_info == NULL)
-	{
-	    g_print (_("Unable to retrieve thumbnail (appears to be on iPod, but no image info available): type: %d, filename: '%s'\n"),
-		     thumb->type, thumb->filename);
-	    return NULL;
-	}
-
-	pixels = itdb_thumb_get_rgb_data (device, thumb);
+        if (item->format == NULL) {
+	    g_warning (_("Unable to retrieve thumbnail (appears to be on iPod, but no image info available): filename: '%s'\n"),
+		       item->filename);
+            return NULL;
+        }
+	pixels = itdb_thumb_get_rgb_data (device, item);
 	if (pixels == NULL)
 	{
 	    return NULL;
 	}
 
+	/* FIXME: this is broken for non-16bpp image formats :-/ */
+	rowstride = get_aligned_width (img_info, sizeof(guint16))*3;
 	pixbuf_full =
 	    gdk_pixbuf_new_from_data (pixels,
 				      GDK_COLORSPACE_RGB,
 				      FALSE, 8,
 				      img_info->width, img_info->height,
-				      img_info->width*3,
+				      rowstride,
 				      (GdkPixbufDestroyNotify)g_free,
 				      NULL);
+
 	/* !! do not g_free(pixels) here: it will be freed when doing a
-	 * gdk_pixbuf_unref() on the GdkPixbuf !! */
+	 * g_object_unref() on the GdkPixbuf !! */
 
 	/* Remove padding from the pixmap and/or cut the pixmap to the
 	   right size. */
@@ -1127,115 +882,46 @@ itdb_thumb_get_gdk_pixbuf (Itdb_Device *device, Itdb_Thumb *thumb)
 					       pad_x, pad_y,
 					       width, height);
 	pixbuf = gdk_pixbuf_copy (pixbuf_sub);
-	gdk_pixbuf_unref (pixbuf_full);
-	gdk_pixbuf_unref (pixbuf_sub);
-    }
+	g_object_unref (pixbuf_full);
+	g_object_unref (pixbuf_sub);
 
-    return pixbuf;
-#else
-    return NULL;
-#endif
+        return pixbuf;
 }
+#else
+gpointer itdb_thumb_ipod_item_to_pixbuf (Itdb_Device *device,
+                                         Itdb_Thumb_Ipod_Item *item)
+{
+    return NULL;
+}
+#endif
 
 /**
- * itdb_thumb_new:
- * 
- * Creates a new #Itdb_Thumb
+ * itdb_artwork_get_pixbuf:
+ * @artwork:    an #Itdb_Artwork
+ * @device:     an #Itdb_Device
+ * @width:      width of the pixbuf to retrieve, -1 for the biggest
+ *              possible size and 0 for the smallest possible size (with no
+ *              scaling)
+ * @height:     height of the pixbuf to retrieve, -1 for the biggest possible
+ *              size and 0 for the smallest possible size (with no scaling)
  *
- * Return Value: newly allocated #Itdb_Thumb to be freed with itdb_thumb_free()
- * after use
- **/
-Itdb_Thumb *itdb_thumb_new (void)
-{
-    Itdb_Thumb *thumb = g_new0 (Itdb_Thumb, 1);
-    return thumb;
-}
-
-/** 
- * itdb_thumb_free:
- * @thumb: an #Itdb_Thumb
+ * Returns a #GdkPixbuf representing the thumbnail stored in @artwork
+ * scaling it if appropriate. If either height or width is -1, then the
+ * biggest unscaled thumbnail available will be returned
  *
- * Frees the memory used by @thumb
- **/
-void itdb_thumb_free (Itdb_Thumb *thumb)
-{
-    g_return_if_fail (thumb);
-
-    g_free (thumb->image_data);
-#ifdef HAVE_GDKPIXBUF
-    if (thumb->pixbuf) {
-        g_object_unref (G_OBJECT (thumb->pixbuf));
-    }
-#endif
-    g_free (thumb->filename);
-    g_free (thumb);
-}
-
-
-/** 
- * itdb_thumb_duplicate:
- * @thumb: an #Itdb_Thumb
+ * Returns: a #GdkPixbuf that must be unreffed when no longer used, NULL
+ * if no artwork could be found or if libgpod is compiled without GdkPixbuf
+ * support
  *
- * Duplicates the data contained in @thumb
- *
- * Return value: a newly allocated copy of @thumb to be freed with 
- * itdb_thumb_free() after use
- **/
-Itdb_Thumb *itdb_thumb_duplicate (Itdb_Thumb *thumb)
+ * Since: 0.7.0
+ */
+gpointer itdb_artwork_get_pixbuf (Itdb_Device *device, Itdb_Artwork *artwork,
+                                  gint width, gint height)
 {
-    Itdb_Thumb *new_thumb;
-
-    g_return_val_if_fail (thumb, NULL);
-
-    new_thumb = itdb_thumb_new ();
-    memcpy (new_thumb, thumb, sizeof (Itdb_Thumb));
-    new_thumb->filename = g_strdup (thumb->filename);
- 
-    if (thumb->image_data)
-    {
-	/* image_data_len already copied with the memcpy above */
-	new_thumb->image_data = g_malloc (thumb->image_data_len);
-	memcpy (new_thumb->image_data, thumb->image_data,
-		new_thumb->image_data_len);
+    g_return_val_if_fail (artwork != NULL, NULL);
+    if (artwork->thumbnail == NULL) {
+        return NULL;
     }
-#ifdef HAVE_GDKPIXBUF
-    if (thumb->pixbuf) {
-        g_object_ref (G_OBJECT (thumb->pixbuf));
-    }
-#endif
-    return new_thumb;
-}
-
-
-G_GNUC_INTERNAL gint
-itdb_thumb_get_byteorder (const ItdbThumbFormat format)
-{
-    switch (format)
-    {
-    case THUMB_FORMAT_UYVY_LE:
-    case THUMB_FORMAT_I420_LE:
-    case THUMB_FORMAT_RGB565_LE:
-    case THUMB_FORMAT_RGB565_LE_90:
-    case THUMB_FORMAT_RGB555_LE:
-    case THUMB_FORMAT_RGB555_LE_90:
-    case THUMB_FORMAT_RGB888_LE:
-    case THUMB_FORMAT_RGB888_LE_90:
-    case THUMB_FORMAT_REC_RGB555_LE:
-    case THUMB_FORMAT_REC_RGB555_LE_90:
-    case THUMB_FORMAT_EXPERIMENTAL_LE:
-	return G_LITTLE_ENDIAN;
-    case THUMB_FORMAT_UYVY_BE:
-    case THUMB_FORMAT_I420_BE:
-    case THUMB_FORMAT_RGB565_BE:
-    case THUMB_FORMAT_RGB565_BE_90:
-    case THUMB_FORMAT_RGB555_BE:
-    case THUMB_FORMAT_RGB555_BE_90:
-    case THUMB_FORMAT_RGB888_BE:
-    case THUMB_FORMAT_RGB888_BE_90:
-    case THUMB_FORMAT_REC_RGB555_BE:
-    case THUMB_FORMAT_REC_RGB555_BE_90:
-    case THUMB_FORMAT_EXPERIMENTAL_BE:
-	return G_BIG_ENDIAN;
-    }
-    g_return_val_if_reached (-1);
+    return itdb_thumb_to_pixbuf_at_size (device, artwork->thumbnail,
+                                         width, height);
 }
