@@ -717,7 +717,7 @@ static const ItdbSerialToModel serial_to_model_mapping[] = {
     { "SZT", "A005" },
     { "TJT", "A099" },
     { "TJU", "A099" },
-    { "TK2", "A107" },
+    { "TK2", "A107" }, /* 4GB Black Nano 1g */
     { "TK3", "A107" },
     { "VQ5", "A477" },
     { "VQ6", "A477" },
@@ -1669,7 +1669,7 @@ const gchar *itdb_info_get_ipod_model_name_string (Itdb_IpodModel model)
        verify if @model is valid */
     while (ipod_model_name_table[i])
     {
-	if (i==model)  return gettext (ipod_model_name_table[i]);
+	if (i==model)  return _(ipod_model_name_table[i]);
 	++i;
     }
     return NULL;
@@ -1697,7 +1697,7 @@ const gchar *itdb_info_get_ipod_generation_string (Itdb_IpodGeneration generatio
     while (ipod_generation_name_table[i])
     {
 	if (i==generation)
-	    return gettext (ipod_generation_name_table[i]);
+	    return _(ipod_generation_name_table[i]);
 	++i;
     }
     return NULL;
