@@ -479,26 +479,26 @@ static GList *parse_one_formats_list (GHashTable *sysinfo_dict,
 {
     GValue *to_parse;
     GList *formats = NULL;
-    GValueArray *array;
+    GArray *array;
     gint i;
 
     to_parse = g_hash_table_lookup (sysinfo_dict, key);
     if (to_parse == NULL) {
         return NULL;
     }
-    if (!G_VALUE_HOLDS (to_parse, G_TYPE_VALUE_ARRAY)) {
+    if (!G_VALUE_HOLDS (to_parse, G_TYPE_ARRAY)) {
         return NULL;
     }
-    array = (GValueArray*)g_value_get_boxed (to_parse);
-    for (i = 0; i < array->n_values; i++) {
+    array = (GArray*)g_value_get_boxed (to_parse);
+    for (i = 0; i < array->len; i++) {
         Itdb_ArtworkFormat *format;
 	/* SysInfoExtended on the iPhone has <string> fields in the artwork
 	 * format array in addition to the hash we parse
 	 */
-	if (!G_VALUE_HOLDS (g_value_array_get_nth (array, i), G_TYPE_HASH_TABLE)) {
+	if (!G_VALUE_HOLDS (&g_array_index (array, GValue, i), G_TYPE_HASH_TABLE)) {
 	    continue;
 	}
-	format = g_value_to_image_format (g_value_array_get_nth (array, i));
+	format = g_value_to_image_format (&g_array_index (array, GValue, i));
 	if (format != NULL) {
 		formats = g_list_prepend (formats, format);
 	}
