@@ -1,7 +1,7 @@
 /*
 |  Copyright (C) 2002-2007 Jorg Schuler <jcsjcs at users sourceforge net>
 |  Part of the gtkpod project.
-| 
+|
 |  URL: http://www.gtkpod.org/
 |  URL: http://gtkpod.sourceforge.net/
 |
@@ -23,7 +23,7 @@
 |
 |  This product is not supported/written/published by Apple!
 |
-|  $Id: itdb_track.c 1718 2007-10-05 19:20:01Z teuf $
+|  $Id$
 */
 
 #include <config.h>
@@ -32,62 +32,46 @@
 #include "itdb_device.h"
 #include <string.h>
 
-static gboolean is_video_ipod (Itdb_Device *device) 
-{
-    const Itdb_IpodInfo *info;
-
-    g_return_val_if_fail (device, FALSE);
-
-    info = itdb_device_get_ipod_info (device);
-
-    if (!info) return FALSE;
-
-    if ((info->ipod_model == ITDB_IPOD_MODEL_VIDEO_WHITE) ||
-	(info->ipod_model == ITDB_IPOD_MODEL_VIDEO_BLACK))
-	return TRUE;
-    else
-	return FALSE;
-}
-
-
 /**
  * itdb_track_new:
- * 
- * Creates an empty #Itdb_Track 
  *
- * Return Value: the new #Itdb_Track, free it with itdb_track_free() when no
+ * Creates an empty #Itdb_Track
+ *
+ * Returns: the new #Itdb_Track, free it with itdb_track_free() when no
  * longer needed
- **/
+ */
 Itdb_Track *itdb_track_new (void)
 {
     Itdb_Track *track = g_new0 (Itdb_Track, 1);
 
     track->artwork = itdb_artwork_new ();
+    track->chapterdata = itdb_chapterdata_new ();
+    track->priv = g_new0 (Itdb_Track_Private, 1);
 
     track->visible = 1;
     return track;
 }
 
+static gboolean haystack (gchar *filetype, gchar **desclist)
+{
+    gchar **dlp;
+    if (!filetype || !desclist) return FALSE;
+    for (dlp=desclist; *dlp; ++dlp)
+    {
+	    if (strstr (filetype, *dlp)) return TRUE;
+    }
+    return FALSE;
+}
+
 /* Attempt to set some of the unknowns to reasonable defaults */
 static void itdb_track_set_defaults (Itdb_Track *tr)
 {
-    auto gboolean haystack (gchar *filetype, gchar **desclist);
-    gboolean haystack (gchar *filetype, gchar **desclist)
-    {
-	gchar **dlp;
-	if (!filetype || !desclist) return FALSE;
-	for (dlp=desclist; *dlp; ++dlp)
-	{
-	    if (strstr (filetype, *dlp)) return TRUE;
-	}
-	return FALSE;
-    }
-
     gchar *mp3_desc[] = {"MPEG", "MP3", "mpeg", "mp3", NULL};
-    gchar *mp4_desc[] = {"AAC", "MP4", "aac", "mp4", NULL};
+    gchar *mp4_desc[] = {"AAC", "MP4", "M4A", "aac", "mp4", "m4a", NULL};
     gchar *audible_subdesc[] = {"Audible", "audible", "Book", "book", NULL};
     gchar *wav_desc[] = {"WAV", "wav", NULL};
     gchar *m4v_desc[] = {"M4V", "MP4", "MP4V", "m4v", "mp4", "mp4v", NULL};
+    gchar *mov_desc[] = {"MOV", "mov", NULL};
 
     g_return_if_fail (tr);
     g_return_if_fail (tr->itdb);
@@ -159,7 +143,7 @@ static void itdb_track_set_defaults (Itdb_Track *tr)
 	    tr->unk144 = 0x0000;  /* default value */
 	}
     }
-    if (is_video_ipod (tr->itdb->device))
+    if (itdb_device_supports_video (tr->itdb->device))
     {
 	/* The unk208 field seems to denote whether the file is a
 	   video or not.  It seems that it must be set to 0x00000002
@@ -170,7 +154,8 @@ static void itdb_track_set_defaults (Itdb_Track *tr)
 	 */
 	if (tr->mediatype == 0)
 	{
-	    if (haystack (tr->filetype, m4v_desc))
+	    if (haystack (tr->filetype, m4v_desc)
+		 || haystack (tr->filetype, mov_desc))
 	    {
 		/* set type to video (0x00000002) */
 		tr->mediatype = 0x00000002;
@@ -209,20 +194,18 @@ static void itdb_track_set_defaults (Itdb_Track *tr)
     }
     if (tr->dbid2 == 0)  tr->dbid2 = tr->dbid;
 }
-    
-
 
 /**
  * itdb_track_add:
- * @itdb: an #Itdb_iTunesDB
- * @track: an #Itdb_Track
- * @pos: position of the track to add
- * 
+ * @itdb:   an #Itdb_iTunesDB
+ * @track:  an #Itdb_Track
+ * @pos:    position of the track to add
+ *
  * Adds @track to @itdb->tracks at position @pos (or at the end if pos
  * is -1). The application is responsible to also add it to the master
- * playlist. The @itdb gets ownership of the @track and will take care of 
+ * playlist. The @itdb gets ownership of the @track and will take care of
  * freeing the memory it uses when it's no longer necessary.
- **/
+ */
 void itdb_track_add (Itdb_iTunesDB *itdb, Itdb_Track *track, gint32 pos)
 {
     g_return_if_fail (itdb);
@@ -240,8 +223,8 @@ void itdb_track_add (Itdb_iTunesDB *itdb, Itdb_Track *track, gint32 pos)
  * itdb_track_free:
  * @track: an #Itdb_Track
  *
- * Frees the memory used by @track 
- **/
+ * Frees the memory used by @track
+ */
 void itdb_track_free (Itdb_Track *track)
 {
     g_return_if_fail (track);
@@ -272,13 +255,14 @@ void itdb_track_free (Itdb_Track *track)
     g_free (track->sort_composer);
     g_free (track->sort_tvshow);
 
-    g_free (track->chapterdata_raw);
+    itdb_chapterdata_free (track->chapterdata);
 
     itdb_artwork_free (track->artwork);
 
     if (track->userdata && track->userdata_destroy)
 	(*track->userdata_destroy) (track->userdata);
 
+    g_free (track->priv);
     g_free (track);
 }
 
@@ -289,7 +273,7 @@ void itdb_track_free (Itdb_Track *track)
  * Removes @track from the #Itdb_iTunesDB it's associated with, and frees the
  * memory it uses. It doesn't remove the track from the playlists it may have
  * been added to, in particular it won't be removed from the master playlist.
- **/
+ */
 void itdb_track_remove (Itdb_Track *track)
 {
     Itdb_iTunesDB *itdb;
@@ -305,12 +289,12 @@ void itdb_track_remove (Itdb_Track *track)
 /**
  * itdb_track_unlink:
  * @track: an #Itdb_Track
- * 
+ *
  * Removes @track from the #Itdb_iTunesDB it's associated with, but do not free
- * memory. It doesn't remove the track from the playlists it may have been 
+ * memory. It doesn't remove the track from the playlists it may have been
  * added to, in particular it won't be removed from the master playlist.
- * track->itdb is set to NULL.
- **/
+ * @track->itdb is set to NULL.
+ */
 void itdb_track_unlink (Itdb_Track *track)
 {
     Itdb_iTunesDB *itdb;
@@ -329,8 +313,8 @@ void itdb_track_unlink (Itdb_Track *track)
  *
  * Duplicates an existing track
  *
- * Return value: a newly allocated #Itdb_Track 
- **/
+ * Returns: a newly allocated #Itdb_Track
+ */
 Itdb_Track *itdb_track_duplicate (Itdb_Track *tr)
 {
     Itdb_Track *tr_dup;
@@ -370,17 +354,16 @@ Itdb_Track *itdb_track_duplicate (Itdb_Track *tr)
     tr_dup->sort_composer = g_strdup (tr->sort_composer);
     tr_dup->sort_tvshow = g_strdup (tr->sort_tvshow);
 
+    /* Copy private data too */
+    tr_dup->priv = g_memdup (tr->priv, sizeof (Itdb_Track_Private));
 
     /* Copy chapterdata */
-    if (tr->chapterdata_raw)
-    {
-	tr_dup->chapterdata_raw = g_new (gchar, tr->chapterdata_raw_length);
-	memcpy (tr_dup->chapterdata_raw, tr->chapterdata_raw,
-		tr->chapterdata_raw_length);
-    }
+    tr_dup->chapterdata = itdb_chapterdata_duplicate (tr->chapterdata);
 
     /* Copy thumbnail data */
-    tr_dup->artwork = itdb_artwork_duplicate (tr->artwork);
+    if (tr->artwork != NULL) {
+        tr_dup->artwork = itdb_artwork_duplicate (tr->artwork);
+    }
 
     /* Copy userdata */
     if (tr->userdata && tr->userdata_duplicate)
@@ -399,65 +382,33 @@ static gboolean itdb_track_set_thumbnails_internal (Itdb_Track *track,
                                                     gpointer *pixbuf,
 						    gint rotation,
 						    GError **error)
-{					     
-    /* FIXME: this looks like it would work, but the problem is that
-     * gtkpod calls this function mostly with tracks that are not yet
-     * part of an iTunesDB. This means only the SMALL and LARGE thumbs
-     * are being added.
-     *
-     * One solution would be to add all kinds of cover thumbs and weed
-     * out the ones not supported when the ArtworkDB is written.
-     *
-     * Suggestions welcome!
-     */
-
+{
     gboolean result = FALSE;
-    ItdbThumbType thumbtypes[] =
-	{ ITDB_THUMB_COVER_SMALL,
-	  ITDB_THUMB_COVER_LARGE,
-	  ITDB_THUMB_COVER_XLARGE,
-	  ITDB_THUMB_COVER_MEDIUM,
-	  ITDB_THUMB_COVER_SMEDIUM,
-	  ITDB_THUMB_COVER_XSMALL,
-	  -1 };
-    ItdbThumbType *thumbtype;
-    const Itdb_ArtworkFormat *formats=NULL;
 
     g_return_val_if_fail (track, FALSE);
     g_return_val_if_fail (filename || image_data || pixbuf, FALSE);
 
-    if (track->itdb && track->itdb->device)
-    {
-	formats = itdb_device_get_artwork_formats (track->itdb->device);
-    }
-
     itdb_artwork_remove_thumbnails (track->artwork);
+    /* clear artwork id */
+    track->artwork->id = 0;
 
-    for (thumbtype=thumbtypes; *thumbtype!=-1; ++thumbtype)
+    if (filename)
     {
-	if (filename)
-	{
-	    result = itdb_artwork_add_thumbnail (track->artwork,
-						 *thumbtype,
-						 filename, rotation, error);
-	}
-	if (image_data)
-	{
-	    result = itdb_artwork_add_thumbnail_from_data (track->artwork,
-							   *thumbtype,
-							   image_data,
-							   image_data_len,
-							   rotation, error);
-	}
-	if (pixbuf)
-	{
-	    result = itdb_artwork_add_thumbnail_from_pixbuf (track->artwork,
-							     *thumbtype,
-							     pixbuf, rotation,
-							     error);
-        }
-	if (result == FALSE)
-	    break;  /* for (thumbtype=...) */
+        result = itdb_artwork_set_thumbnail (track->artwork, filename,
+                                             rotation, error);
+    }
+    if (image_data)
+    {
+        result = itdb_artwork_set_thumbnail_from_data (track->artwork,
+                                                       image_data,
+                                                       image_data_len,
+                                                       rotation, error);
+    }
+    if (pixbuf)
+    {
+        result = itdb_artwork_set_thumbnail_from_pixbuf (track->artwork,
+                                                         pixbuf, rotation,
+                                                         error);
     }
 
     if (result == TRUE)
@@ -483,19 +434,20 @@ static gboolean itdb_track_set_thumbnails_internal (Itdb_Track *track,
     return result;
 }
 
-
 /**
  * itdb_track_set_thumbnails:
- * @track: an #Itdb_Track
+ * @track:    an #Itdb_Track
  * @filename: image file to use as a thumbnail
  *
  * Uses the image contained in @filename to generate iPod thumbnails. The image
  * can be in any format supported by gdk-pixbuf. To save memory, the thumbnails
- * will only be generated when necessary, ie when itdb_save() or a similar 
+ * will only be generated when necessary, i.e. when itdb_save() or a similar
  * function is called.
  *
- * Return value: TRUE if the thumbnail could be added, FALSE otherwise.
- **/
+ * Returns: TRUE if the thumbnail could be added, FALSE otherwise.
+ *
+ * Since: 0.3.0
+ */
 gboolean itdb_track_set_thumbnails (Itdb_Track *track,
 				    const gchar *filename)
 {
@@ -506,23 +458,22 @@ gboolean itdb_track_set_thumbnails (Itdb_Track *track,
 					       0, NULL);
 }
 
-
-
-
 /**
  * itdb_track_set_thumbnails_from_data:
- * @track: an #Itdb_Track
- * @image_data: data used to create the thumbnail (the raw contents of
- *              an image file)
+ * @track:          an #Itdb_Track
+ * @image_data:     data used to create the thumbnail (the raw contents of
+ *                  an image file)
  * @image_data_len: length of above data block
  *
  * Uses @image_data to generate iPod thumbnails. The image can be in
  * any format supported by gdk-pixbuf. To save memory, the thumbnails
- * will only be generated when necessary, ie when itdb_save() or a
+ * will only be generated when necessary, i.e. when itdb_save() or a
  * similar function is called.
  *
- * Return value: TRUE if the thumbnail could be added, FALSE otherwise.
- **/
+ * Returns: TRUE if the thumbnail could be added, FALSE otherwise.
+ *
+ * Since: 0.4.0
+ */
 gboolean itdb_track_set_thumbnails_from_data (Itdb_Track *track,
 					      const guchar *image_data,
 					      gsize image_data_len)
@@ -537,15 +488,17 @@ gboolean itdb_track_set_thumbnails_from_data (Itdb_Track *track,
 
 /**
  * itdb_track_set_thumbnails_from_pixbuf:
- * @track: an #Itdb_Track
+ * @track:  an #Itdb_Track
  * @pixbuf: a #GdkPixbuf used to generate the thumbnail
  *
  * Uses @pixbuf to generate iPod thumbnails. To save memory, the thumbnails
- * will only be generated when necessary, ie when itdb_save() or a
+ * will only be generated when necessary, i.e. when itdb_save() or a
  * similar function is called.
  *
- * Return value: TRUE if the thumbnail could be added, FALSE otherwise.
- **/
+ * Returns: TRUE if the thumbnail could be added, FALSE otherwise.
+ *
+ * Since: 0.5.0
+ */
 gboolean itdb_track_set_thumbnails_from_pixbuf (Itdb_Track *track,
                                                 gpointer pixbuf)
 {
@@ -556,41 +509,45 @@ gboolean itdb_track_set_thumbnails_from_pixbuf (Itdb_Track *track,
                                                pixbuf, 0, NULL);
 }
 
-
 /**
  * itdb_track_remove_thumbnails:
  * @track: an #Itdb_Track
- * 
+ *
  * Removes the thumbnails associated with @track
- **/
+ *
+ * Since: 0.3.0
+ */
 void itdb_track_remove_thumbnails (Itdb_Track *track)
 {
     g_return_if_fail (track);
     itdb_artwork_remove_thumbnails (track->artwork);
     track->artwork_size = 0;
     track->artwork_count = 0;
+    track->mhii_link = 0;
     /* indicate no artwork is present */
     track->has_artwork = 0x02;
 }
 
-
 /**
  * itdb_track_by_id:
  * @itdb: an #Itdb_iTunesDB
- * @id: ID of the track to look for
+ * @id:   ID of the track to look for
+ *
+ * Looks up a track using its ID in @itdb.
  *
- * Looks up a track using its ID in @itdb. 
  * Looking up tracks by ID is not really a good idea because the IDs
  * are created by itdb just before export. The functions are here
  * because they are needed during import of the iTunesDB which is
  * referencing tracks by IDs.
- * This function is very slow (linear in the number of tracks contained in the 
- * database). If you need to lookup many IDs use itdb_track_id_tree_create(), 
- * itdb_track_id_tree_destroy(), and itdb_track_id_tree_by_id().
  *
- * Return value: #Itdb_Track with the ID @id or NULL if the ID cannot be
- * found. 
- **/
+ * This function is very slow (linear in the number of tracks
+ * contained in the database). If you need to lookup many IDs use
+ * itdb_track_id_tree_create(), itdb_track_id_tree_destroy(), and
+ * itdb_track_id_tree_by_id().
+ *
+ * Returns: #Itdb_Track with the ID @id or NULL if the ID cannot be
+ * found.
+ */
 Itdb_Track *itdb_track_by_id (Itdb_iTunesDB *itdb, guint32 id)
 {
     GList *gl;
@@ -614,17 +571,16 @@ static gint track_id_compare (gconstpointer a, gconstpointer b)
     return -1;
 }
 
-
 /**
  * itdb_track_id_tree_create:
  * @itdb: an #Itdb_iTunesDB
  *
  * Creates a balanced-binary tree for quick ID lookup that is used in
- * itdb_track_by_id_tree() function below 
+ * itdb_track_by_id_tree()
  *
- * Return value: a #GTree indexed by track IDs to be freed with 
+ * Returns: a #GTree indexed by track IDs to be freed with
  * itdb_track_id_tree_destroy() when no longer used
- **/
+ */
 GTree *itdb_track_id_tree_create (Itdb_iTunesDB *itdb)
 {
     GTree *idtree;
@@ -647,8 +603,8 @@ GTree *itdb_track_id_tree_create (Itdb_iTunesDB *itdb)
  * itdb_track_id_tree_destroy:
  * @idtree: a #GTree
  *
- * Frees the memory used by @idtree 
- **/
+ * Frees the memory used by @idtree
+ */
 void itdb_track_id_tree_destroy (GTree *idtree)
 {
     g_return_if_fail (idtree);
@@ -659,14 +615,14 @@ void itdb_track_id_tree_destroy (GTree *idtree)
 /**
  * itdb_track_id_tree_by_id:
  * @idtree: a #GTree created using itdb_track_id_tree_create()
- * @id: the ID of the track to search for
+ * @id:     the ID of the track to search for
  *
- * Lookup an #Itdb_Track by @id using @idtree for faster lookup (compared to
- * itdb_track_by_id)
+ * Lookup an #Itdb_Track by @id using @idtree for faster lookup
+ * (compared to itdb_track_by_id())
  *
- * Return value: the #Itdb_Track whose ID is @id, or NULL if such a track 
+ * Returns: the #Itdb_Track whose ID is @id, or NULL if such a track
  * couldn't be found
- **/
+ */
 Itdb_Track *itdb_track_id_tree_by_id (GTree *idtree, guint32 id)
 {
     g_return_val_if_fail (idtree, NULL);
@@ -674,4 +630,53 @@ Itdb_Track *itdb_track_id_tree_by_id (GTree *idtree, guint32 id)
     return (Itdb_Track *)g_tree_lookup (idtree, &id);
 }
 
+/**
+ * itdb_track_has_thumbnails:
+ * @track: an #Itdb_Track
+ *
+ * Determine if a @track has thumbnails
+ *
+ * Returns: TRUE if @track has artwork available, FALSE otherwise
+ *
+ * Since: 0.7.0
+ */
+gboolean itdb_track_has_thumbnails (Itdb_Track *track)
+{
+    g_return_val_if_fail (track != NULL, FALSE);
+    return ((track->artwork) && (track->artwork->thumbnail));
+}
+
+/**
+ * itdb_track_get_thumbnail:
+ * @track:  an #Itdb_Track
+ * @width:  width of the pixbuf to retrieve, -1 for the biggest possible size
+ *          (with no scaling)
+ * @height: height of the pixbuf to retrieve, -1 for the biggest possible size
+ *          (with no scaling)
+ *
+ * Get a thumbnail representing the cover associated with the current track,
+ * scaling it if appropriate. If either height or width is -1, then the biggest
+ * unscaled thumbnail available will be returned.
+ *
+ * Returns: a #GdkPixbuf that must be unreffed when no longer used, NULL
+ * if no artwork could be found or if libgpod is compiled without GdkPixbuf
+ * support
+ *
+ * Since: 0.7.0
+ */
+gpointer itdb_track_get_thumbnail (Itdb_Track *track, gint width, gint height)
+{
+    g_return_val_if_fail (track != NULL, NULL);
+    if (!itdb_track_has_thumbnails (track)) {
+        return NULL;
+    }
+    if (track->itdb != NULL) {
+        return itdb_thumb_to_pixbuf_at_size (track->itdb->device,
+                                             track->artwork->thumbnail,
+                                             width, height);
+    } else {
+        return itdb_thumb_to_pixbuf_at_size (NULL, track->artwork->thumbnail,
+                                             width, height);
+    }
+}
 
