@@ -19,7 +19,7 @@
  *
  * This product is not supported/written/published by Apple!
  *
- * $Id: test-init-ipod.c,v 1.4 2006/10/02 13:17:58 jcsjcs Exp $
+ * $Id$
  *
  */
 
@@ -53,7 +53,9 @@ main (int argc, char **argv)
         return 1;
     }
     setlocale (LC_ALL, "");
+#if !GLIB_CHECK_VERSION(2, 36, 0)
     g_type_init ();
+#endif
 
     if (argc == 3)
     {
