@@ -18,7 +18,7 @@
  *
  * This product is not supported/written/published by Apple!
  *
- * $Id: test-photos.c,v 1.13 2007/05/07 11:42:09 jcsjcs Exp $
+ * $Id$
  *
  */
 
@@ -59,41 +59,32 @@ static Itdb_Artwork *get_photo_by_id (Itdb_PhotoDB *db, guint32 id)
     return NULL;
 }
 
-static void
-save_itdb_thumb (Itdb_PhotoDB *itdb, Itdb_Thumb *thumb,
-		 const gchar *filename)
-{
-	GdkPixbuf *pixbuf;
-	
-	pixbuf = itdb_thumb_get_gdk_pixbuf (itdb->device, thumb);
-
-	if (pixbuf != NULL) {
-		gdk_pixbuf_save (pixbuf, filename, "png", NULL, NULL);
-		gdk_pixbuf_unref (pixbuf);
-	}
-}
-
 static void
 dump_thumbs (Itdb_PhotoDB *db, Itdb_Artwork *artwork,
 	     const gchar *album_name, const gchar *dir)
 {
 	GList *it;
 	gint i = 0;
+        GList *thumbnails;
 
-	for (it = artwork->thumbnails; it != NULL; it = it->next, i++) {
-		Itdb_Thumb *thumb;
+        thumbnails = itdb_thumb_to_pixbufs (db->device, artwork->thumbnail);
+	for (it = thumbnails; it != NULL; it = it->next, i++) {
 		gchar *filename, *path;
+                GdkPixbuf *pixbuf;
 
-		thumb = (Itdb_Thumb *)it->data;
-		g_return_if_fail (thumb);
+                pixbuf = GDK_PIXBUF (it->data);
+
+		g_return_if_fail (pixbuf);
 
 		filename = g_strdup_printf ("%s-%d-%d.png",
 					    album_name, artwork->id, i );
 		path = g_build_filename (dir, filename, NULL);
 		g_free (filename);
-		save_itdb_thumb (db, thumb, path);
+		gdk_pixbuf_save (pixbuf, path, "png", NULL, NULL);
+		g_object_unref (pixbuf);
 		g_free (path);
 	}
+        g_list_free (thumbnails);
 }
 
 static void
@@ -388,7 +379,9 @@ main (int argc, char **argv)
 	return 1;
     }
     setlocale (LC_ALL, "");
+#if !GLIB_CHECK_VERSION(2, 36, 0)
     g_type_init ();
+#endif
 
     if (strcmp (argv[1], "dump") == 0)
     {
